# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


"""
Command-line interface to the OpenStack Profiler.
"""

import inspect
import sys

import argparse

import osprofiler
from osprofiler.cmd import cliutils
from osprofiler.cmd import commands
from osprofiler.cmd import exc


class OSProfilerShell(object):

    def __init__(self, argv):
        args = self._get_base_parser().parse_args(argv)

        if not (args.os_auth_token and args.ceilometer_url):
            if not args.os_username:
                raise exc.CommandError(
                    "You must provide a username via either --os-username or "
                    "via env[OS_USERNAME]")

            if not args.os_password:
                raise exc.CommandError(
                    "You must provide a password via either --os-password or "
                    "via env[OS_PASSWORD]")

            if self._no_project_and_domain_set(args):
                # steer users towards Keystone V3 API
                raise exc.CommandError(
                    "You must provide a project_id via either --os-project-id "
                    "or via env[OS_PROJECT_ID] and a domain_name via either "
                    "--os-user-domain-name or via env[OS_USER_DOMAIN_NAME] or "
                    "a domain_id via either --os-user-domain-id or via "
                    "env[OS_USER_DOMAIN_ID]")

            if not args.os_auth_url:
                raise exc.CommandError(
                    "You must provide an auth url via either --os-auth-url or "
                    "via env[OS_AUTH_URL]")

        args.func(args)

    def _get_base_parser(self):
        parser = argparse.ArgumentParser(
            prog="osprofiler",
            description=__doc__.strip(),
            add_help=True
        )

        parser.add_argument("-v", "--version",
                            action="version",
                            version=osprofiler.__version__)

        self._append_ceilometer_args(parser)
        self._append_identity_args(parser)
        self._append_subcommands(parser)

        return parser

    def _append_ceilometer_args(self, parent_parser):
        parser = parent_parser.add_argument_group("ceilometer")
        parser.add_argument(
            "--ceilometer-url", default=cliutils.env("CEILOMETER_URL"),
            help="Defaults to env[CEILOMETER_URL].")
        parser.add_argument(
            "--ceilometer-api-version",
            default=cliutils.env("CEILOMETER_API_VERSION", default="2"),
            help="Defaults to env[CEILOMETER_API_VERSION] or 2.")

    def _append_identity_args(self, parent_parser):
        # FIXME(fabgia): identity related parameters should be passed by the
        # Keystone client itself to avoid constant update in all the services
        # clients. When this fix is merged this method can be made obsolete.
        # Bug: https://bugs.launchpad.net/python-keystoneclient/+bug/1332337
        parser = parent_parser.add_argument_group("identity")
        parser.add_argument("-k", "--insecure",
                            default=False,
                            action="store_true",
                            help="Explicitly allow osprofiler to "
                            "perform \"insecure\" SSL (https) requests. "
                            "The server's certificate will "
                            "not be verified against any certificate "
                            "authorities. This option should be used with "
                            "caution.")

        # User related options
        parser.add_argument("--os-username",
                            default=cliutils.env("OS_USERNAME"),
                            help="Defaults to env[OS_USERNAME].")

        parser.add_argument("--os-user-id",
                            default=cliutils.env("OS_USER_ID"),
                            help="Defaults to env[OS_USER_ID].")

        parser.add_argument("--os-password",
                            default=cliutils.env("OS_PASSWORD"),
                            help="Defaults to env[OS_PASSWORD].")

        # Domain related options
        parser.add_argument("--os-user-domain-id",
                            default=cliutils.env("OS_USER_DOMAIN_ID"),
                            help="Defaults to env[OS_USER_DOMAIN_ID].")

        parser.add_argument("--os-user-domain-name",
                            default=cliutils.env("OS_USER_DOMAIN_NAME"),
                            help="Defaults to env[OS_USER_DOMAIN_NAME].")

        parser.add_argument("--os-project-domain-id",
                            default=cliutils.env("OS_PROJECT_DOMAIN_ID"),
                            help="Defaults to env[OS_PROJECT_DOMAIN_ID].")

        parser.add_argument("--os-project-domain-name",
                            default=cliutils.env("OS_PROJECT_DOMAIN_NAME"),
                            help="Defaults to env[OS_PROJECT_DOMAIN_NAME].")

        # Project V3 or Tenant V2 related options
        parser.add_argument("--os-project-id",
                            default=cliutils.env("OS_PROJECT_ID"),
                            help="Another way to specify tenant ID. "
                                 "This option is mutually exclusive with "
                                 " --os-tenant-id. "
                                 "Defaults to env[OS_PROJECT_ID].")

        parser.add_argument("--os-project-name",
                            default=cliutils.env("OS_PROJECT_NAME"),
                            help="Another way to specify tenant name. "
                                 "This option is mutually exclusive with "
                                 " --os-tenant-name. "
                                 "Defaults to env[OS_PROJECT_NAME].")

        parser.add_argument("--os-tenant-id",
                            default=cliutils.env("OS_TENANT_ID"),
                            help="This option is mutually exclusive with "
                                 " --os-project-id. "
                                 "Defaults to env[OS_PROJECT_ID].")

        parser.add_argument("--os-tenant-name",
                            default=cliutils.env("OS_TENANT_NAME"),
                            help="Defaults to env[OS_TENANT_NAME].")

        # Auth related options
        parser.add_argument("--os-auth-url",
                            default=cliutils.env("OS_AUTH_URL"),
                            help="Defaults to env[OS_AUTH_URL].")

        parser.add_argument("--os-auth-token",
                            default=cliutils.env("OS_AUTH_TOKEN"),
                            help="Defaults to env[OS_AUTH_TOKEN].")

        parser.add_argument("--os-cacert",
                            metavar="<ca-certificate-file>",
                            dest="os_cacert",
                            default=cliutils.env("OS_CACERT"),
                            help="Path of CA TLS certificate(s) used to verify"
                            " the remote server\"s certificate. Without this "
                            "option ceilometer looks for the default system CA"
                            " certificates.")

        parser.add_argument("--os-cert",
                            help="Path of certificate file to use in SSL "
                            "connection. This file can optionally be "
                            "prepended with the private key.")

        parser.add_argument("--os-key",
                            help="Path of client key to use in SSL "
                            "connection. This option is not necessary "
                            "if your key is prepended to your cert file.")

        # Service Catalog related options
        parser.add_argument("--os-service-type",
                            default=cliutils.env("OS_SERVICE_TYPE"),
                            help="Defaults to env[OS_SERVICE_TYPE].")

        parser.add_argument("--os-endpoint-type",
                            default=cliutils.env("OS_ENDPOINT_TYPE"),
                            help="Defaults to env[OS_ENDPOINT_TYPE].")

        parser.add_argument("--os-region-name",
                            default=cliutils.env("OS_REGION_NAME"),
                            help="Defaults to env[OS_REGION_NAME].")

    def _append_subcommands(self, parent_parser):
        subcommands = parent_parser.add_subparsers(help="<subcommands>")
        for group_cls in commands.BaseCommand.__subclasses__():
            group_parser = subcommands.add_parser(group_cls.group_name)
            subcommand_parser = group_parser.add_subparsers()

            for name, callback in inspect.getmembers(
                    group_cls(), predicate=inspect.ismethod):
                command = name.replace("_", "-")
                desc = callback.__doc__ or ""
                help_message = desc.strip().split("\n")[0]
                arguments = getattr(callback, "arguments", [])

                command_parser = subcommand_parser.add_parser(
                    command, help=help_message, description=desc)
                for (args, kwargs) in arguments:
                    command_parser.add_argument(*args, **kwargs)
                command_parser.set_defaults(func=callback)

    def _no_project_and_domain_set(self, args):
        if not (args.os_project_id or (args.os_project_name and
                (args.os_user_domain_name or args.os_user_domain_id)) or
                (args.os_tenant_id or args.os_tenant_name)):
            return True
        else:
            return False


def main(args=None):
    if args is None:
        args = sys.argv[1:]

    try:
        OSProfilerShell(args)
    except exc.CommandError as e:
        print (e.message)
        return 1


if __name__ == "__main__":
    main()



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from osprofiler._notifiers import base
from osprofiler.tests import test


class NotifierBaseTestCase(test.TestCase):

    def test_factory(self):

        class A(base.Notifier):

            def notify(self, a):
                return a

        self.assertEqual(base.Notifier.factory("A")(10), 10)

    def test_factory_with_args(self):

        class B(base.Notifier):

            def __init__(self, a, b=10):
                self.a = a
                self.b = b

            def notify(self, c):
                return self.a + self.b + c

        self.assertEqual(base.Notifier.factory("B", 5, b=7)(10), 22)

    def test_factory_not_found(self):
        self.assertRaises(TypeError, base.Notifier.factory, "non existing")

    def test_notify(self):
        base.Notifier().notify("")

    def test_plugins_are_imported(self):
        base.Notifier.factory("Messaging", mock.MagicMock(), "context",
                              "transport", "project", "service", "host")



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import os

from osprofiler.cmd import cliutils
from osprofiler.cmd import exc
from osprofiler.parsers import ceilometer as ceiloparser


class BaseCommand(object):
    group_name = None


class TraceCommands(BaseCommand):
    group_name = "trace"

    @cliutils.arg("trace", help="File with trace or trace id")
    @cliutils.arg("--json", dest="use_json", action="store_true",
                  help="show trace in JSON")
    @cliutils.arg("--html", dest="use_html", action="store_true",
                  help="show trace in HTML")
    @cliutils.arg("--out", dest="file_name", help="save output in file")
    def show(self, args):
        """Displays trace-results by given trace id in HTML or JSON format."""

        trace = None

        if os.path.exists(args.trace):
            trace = json.load(open(args.trace))
        else:
            try:
                import ceilometerclient.client
                import ceilometerclient.exc
                import ceilometerclient.shell
            except ImportError:
                raise ImportError(
                    "To use this command, you should install "
                    "'ceilometerclient' manually. Use command:\n "
                    "'pip install ceilometerclient'.")
            try:
                client = ceilometerclient.client.get_client(
                    args.ceilometer_api_version, **args.__dict__)
                notifications = ceiloparser.get_notifications(
                    client, args.trace)
            except Exception as e:
                if hasattr(e, "http_status") and e.http_status == 401:
                    msg = "Invalid OpenStack Identity credentials."
                else:
                    msg = "Something has gone wrong. See logs for more details"
                raise exc.CommandError(msg)

            if notifications:
                trace = ceiloparser.parse_notifications(notifications)

        if not trace:
            msg = ("Trace with UUID %s not found. "
                   "There are 3 possible reasons: \n"
                   " 1) You are using not admin credentials\n"
                   " 2) You specified wrong trace id\n"
                   " 3) You specified wrong HMAC Key in original calling"
                   % args.trace)
            raise exc.CommandError(msg)

        if args.use_json:
            output = json.dumps(trace)
        elif args.use_html:
            with open(os.path.join(os.path.dirname(__file__),
                                   "template.html")) as html_template:
                output = html_template.read().replace(
                    "$DATA", json.dumps(trace, indent=2))
        else:
            raise exc.CommandError("You should choose one of the following "
                                   "output-formats: --json or --html.")

        if args.file_name:
            with open(args.file_name, "w+") as output_file:
                output_file.write(output)
        else:
            print (output)



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from osprofiler import sqlalchemy
from osprofiler.tests import test


class SqlalchemyTracingTestCase(test.TestCase):

    @mock.patch("osprofiler.sqlalchemy.profiler")
    def test_before_execute(self, mock_profiler):
        handler = sqlalchemy._before_cursor_execute("sql")

        handler(mock.MagicMock(), 1, 2, 3, 4, 5)
        expected_info = {"db": {"statement": 2, "params": 3}}
        mock_profiler.start.assert_called_once_with("sql", info=expected_info)

    @mock.patch("osprofiler.sqlalchemy.profiler")
    def test_after_execute(self, mock_profiler):
        handler = sqlalchemy._after_cursor_execute()
        handler(mock.MagicMock(), 1, 2, 3, 4, 5)
        mock_profiler.stop.assert_called_once_with()

    @mock.patch("osprofiler.sqlalchemy._before_cursor_execute")
    @mock.patch("osprofiler.sqlalchemy._after_cursor_execute")
    def test_add_tracing(self, mock_after_exc, mock_before_exc):
        sa = mock.MagicMock()
        engine = mock.MagicMock()

        mock_before_exc.return_value = "before"
        mock_after_exc.return_value = "after"

        sqlalchemy.add_tracing(sa, engine, "sql")

        mock_before_exc.assert_called_once_with("sql")
        mock_after_exc.assert_called_once_with()
        expected_calls = [
            mock.call(engine, "before_cursor_execute", "before"),
            mock.call(engine, "after_cursor_execute", "after")
        ]
        self.assertEqual(sa.event.listen.call_args_list, expected_calls)

    @mock.patch("osprofiler.sqlalchemy._before_cursor_execute")
    @mock.patch("osprofiler.sqlalchemy._after_cursor_execute")
    def test_disable_and_enable(self, mock_after_exc, mock_before_exc):
        sqlalchemy.disable()

        sa = mock.MagicMock()
        engine = mock.MagicMock()
        sqlalchemy.add_tracing(sa, engine, "sql")
        self.assertFalse(mock_after_exc.called)
        self.assertFalse(mock_before_exc.called)

        sqlalchemy.enable()
        sqlalchemy.add_tracing(sa, engine, "sql")
        self.assertTrue(mock_after_exc.called)
        self.assertTrue(mock_before_exc.called)



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime


def _build_tree(nodes):
    """Builds the tree (forest) data structure based on the list of nodes.

   Works in O(n).

   :param nodes: list of nodes, where each node is a dictionary with fields
                 "parent_id", "trace_id", "info"
   :returns: list of top level ("root") nodes in form of dictionaries,
             each containing the "info" and "children" fields, where
             "children" is the list of child nodes ("children" will be
             empty for leafs)
   """

    tree = []

    for trace_id in nodes:
        node = nodes[trace_id]
        node.setdefault("children", [])
        parent_id = node["parent_id"]
        if parent_id in nodes:
            nodes[parent_id].setdefault("children", [])
            nodes[parent_id]["children"].append(node)
        else:
            tree.append(node)  # no parent => top-level node

    for node in nodes:
        nodes[node]["children"].sort(key=lambda x: x["info"]["started"])

    return sorted(tree, key=lambda x: x["info"]["started"])


def parse_notifications(notifications):
    """Parse & builds tree structure from list of ceilometer notifications."""

    result = {}
    started_at = 0
    finished_at = 0

    for n in notifications:
        traits = n["traits"]

        def find_field(f_name):
            return [t["value"] for t in traits if t["name"] == f_name][0]

        trace_id = find_field("trace_id")
        parent_id = find_field("parent_id")
        name = find_field("name")
        project = find_field("project")
        service = find_field("service")
        host = find_field("host")
        timestamp = find_field("timestamp")

        timestamp = datetime.datetime.strptime(timestamp,
                                               "%Y-%m-%dT%H:%M:%S.%f")

        if trace_id not in result:
            result[trace_id] = {
                "info": {
                    "name": name.split("-")[0],
                    "project": project,
                    "service": service,
                    "host": host,
                },
                "trace_id": trace_id,
                "parent_id": parent_id,
            }

        result[trace_id]["info"]["meta.raw_payload.%s" % name] = n.get(
            "raw", {}).get("payload", {})

        if name.endswith("stop"):
            result[trace_id]["info"]["finished"] = timestamp
        else:
            result[trace_id]["info"]["started"] = timestamp

        if not started_at or started_at > timestamp:
            started_at = timestamp

        if not finished_at or finished_at < timestamp:
            finished_at = timestamp

    def msec(dt):
        # NOTE(boris-42): Unfortunately this is the simplest way that works in
        #                 py26 and py27
        microsec = (dt.microseconds + (dt.seconds + dt.days * 24 * 3600) * 1e6)
        return int(microsec / 1000.0)

    for r in result.values():
        # NOTE(boris-42): We are not able to guarantee that ceilometer consumed
        #                 all messages => so we should at make duration 0ms.
        if "started" not in r["info"]:
            r["info"]["started"] = r["info"]["finished"]
        if "finished" not in r["info"]:
            r["info"]["finished"] = r["info"]["started"]

        r["info"]["started"] = msec(r["info"]["started"] - started_at)
        r["info"]["finished"] = msec(r["info"]["finished"] - started_at)

    return {
        "info": {
            "name": "total",
            "started": 0,
            "finished": msec(finished_at - started_at) if started_at else 0
        },
        "children": _build_tree(result)
    }


def get_notifications(ceilometer, base_id):
    """Retrieves and parses notification from ceilometer.

    :param ceilometer: Initialized ceilometer client.
    :param base_id: Base id of trace elements.
    """

    _filter = [{"field": "base_id", "op": "eq", "value": base_id}]
    # limit is hardcoded in this code state. Later that will be changed via
    # connection string usage
    return [n.to_dict()
            for n in ceilometer.events.list(_filter, limit=100000)]



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import os
import sys

import mock
import six

from osprofiler.cmd import exc
from osprofiler.cmd import shell
from osprofiler.tests import test


class ShellTestCase(test.TestCase):
    def setUp(self):
        super(ShellTestCase, self).setUp()
        self.old_environment = os.environ.copy()
        os.environ = {
            "OS_USERNAME": "username",
            "OS_USER_ID": "user_id",
            "OS_PASSWORD": "password",
            "OS_USER_DOMAIN_ID": "user_domain_id",
            "OS_USER_DOMAIN_NAME": "user_domain_name",
            "OS_PROJECT_DOMAIN_ID": "project_domain_id",
            "OS_PROJECT_DOMAIN_NAME": "project_domain_name",
            "OS_PROJECT_ID": "project_id",
            "OS_PROJECT_NAME": "project_name",
            "OS_TENANT_ID": "tenant_id",
            "OS_TENANT_NAME": "tenant_name",
            "OS_AUTH_URL": "http://127.0.0.1:5000/v3/",
            "OS_AUTH_TOKEN": "pass",
            "OS_CACERT": "/path/to/cacert",
            "OS_SERVICE_TYPE": "service_type",
            "OS_ENDPOINT_TYPE": "public",
            "OS_REGION_NAME": "test"
        }

        self.ceiloclient = mock.MagicMock()
        sys.modules["ceilometerclient"] = self.ceiloclient
        self.addCleanup(sys.modules.pop, "ceilometerclient", None)
        ceilo_modules = ["client", "exc", "shell"]
        for module in ceilo_modules:
            sys.modules["ceilometerclient.%s" % module] = getattr(
                self.ceiloclient, module)
            self.addCleanup(
                sys.modules.pop, "ceilometerclient.%s" % module, None)

    def tearDown(self):
        super(ShellTestCase, self).tearDown()
        os.environ = self.old_environment

    @mock.patch("sys.stdout", six.StringIO())
    @mock.patch("osprofiler.cmd.shell.OSProfilerShell")
    def test_shell_main(self, mock_shell):
        mock_shell.side_effect = exc.CommandError("some_message")
        shell.main()
        self.assertEqual("some_message\n", sys.stdout.getvalue())

    def run_command(self, cmd):
        shell.OSProfilerShell(cmd.split())

    def _test_with_command_error(self, cmd, expected_message):
        try:
            self.run_command(cmd)
        except exc.CommandError as actual_error:
            self.assertEqual(str(actual_error), expected_message)
        else:
            raise ValueError(
                "Expected: `osprofiler.cmd.exc.CommandError` is raised with "
                "message: '%s'." % expected_message)

    def test_username_is_not_presented(self):
        os.environ.pop("OS_USERNAME")
        msg = ("You must provide a username via either --os-username or "
               "via env[OS_USERNAME]")
        self._test_with_command_error("trace show fake-uuid", msg)

    def test_password_is_not_presented(self):
        os.environ.pop("OS_PASSWORD")
        msg = ("You must provide a password via either --os-password or "
               "via env[OS_PASSWORD]")
        self._test_with_command_error("trace show fake-uuid", msg)

    def test_auth_url(self):
        os.environ.pop("OS_AUTH_URL")
        msg = ("You must provide an auth url via either --os-auth-url or "
               "via env[OS_AUTH_URL]")
        self._test_with_command_error("trace show fake-uuid", msg)

    def test_no_project_and_domain_set(self):
        os.environ.pop("OS_PROJECT_ID")
        os.environ.pop("OS_PROJECT_NAME")
        os.environ.pop("OS_TENANT_ID")
        os.environ.pop("OS_TENANT_NAME")
        os.environ.pop("OS_USER_DOMAIN_ID")
        os.environ.pop("OS_USER_DOMAIN_NAME")

        msg = ("You must provide a project_id via either --os-project-id or "
               "via env[OS_PROJECT_ID] and a domain_name via either "
               "--os-user-domain-name or via env[OS_USER_DOMAIN_NAME] or a "
               "domain_id via either --os-user-domain-id or via "
               "env[OS_USER_DOMAIN_ID]")
        self._test_with_command_error("trace show fake-uuid", msg)

    def test_trace_show_ceilometrclient_is_missed(self):
        sys.modules["ceilometerclient"] = None
        sys.modules["ceilometerclient.client"] = None
        sys.modules["ceilometerclient.exc"] = None
        sys.modules["ceilometerclient.shell"] = None

        self.assertRaises(ImportError, shell.main,
                          "trace show fake_uuid".split())

    def test_trace_show_unauthorized(self):
        class FakeHTTPUnauthorized(Exception):
            http_status = 401

        self.ceiloclient.client.get_client.side_effect = FakeHTTPUnauthorized

        msg = "Invalid OpenStack Identity credentials."
        self._test_with_command_error("trace show fake_id", msg)

    def test_trace_show_unknown_error(self):
        class FakeException(Exception):
            pass

        self.ceiloclient.client.get_client.side_effect = FakeException
        msg = "Something has gone wrong. See logs for more details"
        self._test_with_command_error("trace show fake_id", msg)

    @mock.patch("osprofiler.parsers.ceilometer.get_notifications")
    @mock.patch("osprofiler.parsers.ceilometer.parse_notifications")
    def test_trace_show_no_selected_format(self, mock_notifications, mock_get):
        mock_get.return_value = "some_notificatios"
        msg = ("You should choose one of the following output-formats: "
               "--json or --html.")
        self._test_with_command_error("trace show fake_id", msg)

    @mock.patch("osprofiler.parsers.ceilometer.get_notifications")
    def test_trace_show_trace_id_not_found(self, mock_get):
        mock_get.return_value = None

        fake_trace_id = "fake_id"
        msg = ("Trace with UUID %s not found. There are 3 possible reasons: \n"
               " 1) You are using not admin credentials\n"
               " 2) You specified wrong trace id\n"
               " 3) You specified wrong HMAC Key in original calling"
               % fake_trace_id)

        self._test_with_command_error("trace show %s" % fake_trace_id, msg)

    @mock.patch("sys.stdout", six.StringIO())
    @mock.patch("osprofiler.parsers.ceilometer.get_notifications")
    @mock.patch("osprofiler.parsers.ceilometer.parse_notifications")
    def test_trace_show_in_json(self, mock_notifications, mock_get):
        mock_get.return_value = "some notification"
        notifications = {
            "info": {
                "started": 0, "finished": 0, "name": "total"}, "children": []}
        mock_notifications.return_value = notifications

        self.run_command("trace show fake_id --json")
        self.assertEqual("%s\n" % json.dumps(notifications),
                         sys.stdout.getvalue())

    @mock.patch("sys.stdout", six.StringIO())
    @mock.patch("osprofiler.parsers.ceilometer.get_notifications")
    @mock.patch("osprofiler.parsers.ceilometer.parse_notifications")
    def test_trace_show_in_html(self, mock_notifications, mock_get):
        mock_get.return_value = "some notification"

        notifications = {
            "info": {
                "started": 0, "finished": 0, "name": "total"}, "children": []}
        mock_notifications.return_value = notifications

        # NOTE(akurilin): to simplify assert statement, html-template should be
        # replaced.
        html_template = (
            "A long time ago in a galaxy far, far away..."
            "    some_data = $DATA"
            "It is a period of civil war. Rebel"
            "spaceships, striking from a hidden"
            "base, have won their first victory"
            "against the evil Galactic Empire.")

        with mock.patch("osprofiler.cmd.commands.open",
                        mock.mock_open(read_data=html_template), create=True):
            self.run_command("trace show fake_id --html")
        self.assertEqual("A long time ago in a galaxy far, far away..."
                         "    some_data = %s"
                         "It is a period of civil war. Rebel"
                         "spaceships, striking from a hidden"
                         "base, have won their first victory"
                         "against the evil Galactic Empire."
                         "\n" % json.dumps(notifications, indent=2),
                         sys.stdout.getvalue())

    @mock.patch("sys.stdout", six.StringIO())
    @mock.patch("osprofiler.parsers.ceilometer.get_notifications")
    @mock.patch("osprofiler.parsers.ceilometer.parse_notifications")
    def test_trace_show_write_to_file(self, mock_notifications, mock_get):
        mock_get.return_value = "some notification"
        notifications = {
            "info": {
                "started": 0, "finished": 0, "name": "total"}, "children": []}
        mock_notifications.return_value = notifications

        with mock.patch("osprofiler.cmd.commands.open",
                        mock.mock_open(), create=True) as mock_open:
            self.run_command("trace show fake_id --json --out='/file'")

            output = mock_open.return_value.__enter__.return_value
            output.write.assert_called_once_with(json.dumps(notifications))



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from osprofiler.parsers import ceilometer
from osprofiler.tests import test


class CeilometerParserTestCase(test.TestCase):
    def test_build_empty_tree(self):
        self.assertEqual(ceilometer._build_tree({}), [])

    def test_build_complex_tree(self):
        test_input = {
            "2": {"parent_id": "0", "trace_id": "2", "info": {"started": 1}},
            "1": {"parent_id": "0", "trace_id": "1", "info": {"started": 0}},
            "21": {"parent_id": "2", "trace_id": "21", "info": {"started": 6}},
            "22": {"parent_id": "2", "trace_id": "22", "info": {"started": 7}},
            "11": {"parent_id": "1", "trace_id": "11", "info": {"started": 1}},
            "113": {"parent_id": "11", "trace_id": "113",
                    "info": {"started": 3}},
            "112": {"parent_id": "11", "trace_id": "112",
                    "info": {"started": 2}},
            "114": {"parent_id": "11", "trace_id": "114",
                    "info": {"started": 5}}
        }

        expected_output = [
            {
                "parent_id": "0",
                "trace_id": "1",
                "info": {"started": 0},
                "children": [
                    {
                        "parent_id": "1",
                        "trace_id": "11",
                        "info": {"started": 1},
                        "children": [
                            {"parent_id": "11", "trace_id": "112",
                             "info": {"started": 2}, "children": []},
                            {"parent_id": "11", "trace_id": "113",
                             "info": {"started": 3}, "children": []},
                            {"parent_id": "11", "trace_id": "114",
                             "info": {"started": 5}, "children": []}
                        ]
                    }
                ]
            },
            {
                "parent_id": "0",
                "trace_id": "2",
                "info": {"started": 1},
                "children": [
                    {"parent_id": "2", "trace_id": "21",
                     "info": {"started": 6}, "children": []},
                    {"parent_id": "2", "trace_id": "22",
                     "info": {"started": 7}, "children": []}
                ]
            }
        ]

        self.assertEqual(ceilometer._build_tree(test_input), expected_output)

    def test_parse_notifications_empty(self):
        expected = {
            "info": {
                "name": "total",
                "started": 0,
                "finished": 0
            },
            "children": []
        }
        self.assertEqual(ceilometer.parse_notifications([]), expected)

    def test_parse_notifications(self):
        events = [
            {
                "traits": [
                    {
                        "type": "string",
                        "name": "base_id",
                        "value": "7253ca8c-33b3-4f84-b4f1-f5a4311ddfa4"
                    },
                    {
                        "type": "string",
                        "name": "host",
                        "value": "ubuntu"
                    },
                    {
                        "type": "string",
                        "name": "method",
                        "value": "POST"
                    },
                    {
                        "type": "string",
                        "name": "name",
                        "value": "wsgi-start"
                    },
                    {
                        "type": "string",
                        "name": "parent_id",
                        "value": "7253ca8c-33b3-4f84-b4f1-f5a4311ddfa4"
                    },
                    {
                        "type": "string",
                        "name": "project",
                        "value": "keystone"
                    },
                    {
                        "type": "string",
                        "name": "service",
                        "value": "main"
                    },
                    {
                        "type": "string",
                        "name": "timestamp",
                        "value": "2015-12-23T14:02:22.338776"
                    },
                    {
                        "type": "string",
                        "name": "trace_id",
                        "value": "06320327-2c2c-45ae-923a-515de890276a"
                    }
                ],
                "raw": {},
                "generated": "2015-12-23T10:41:38.415793",
                "event_type": "profiler.main",
                "message_id": "65fc1553-3082-4a6f-9d1e-0e3183f57a47"},
            {
                "traits":
                    [
                        {
                            "type": "string",
                            "name": "base_id",
                            "value": "7253ca8c-33b3-4f84-b4f1-f5a4311ddfa4"
                        },
                        {
                            "type": "string",
                            "name": "host",
                            "value": "ubuntu"
                        },
                        {
                            "type": "string",
                            "name": "name",
                            "value": "wsgi-stop"
                        },
                        {
                            "type": "string",
                            "name": "parent_id",
                            "value": "7253ca8c-33b3-4f84-b4f1-f5a4311ddfa4"
                        },
                        {
                            "type": "string",
                            "name": "project",
                            "value": "keystone"
                        },
                        {
                            "type": "string",
                            "name": "service",
                            "value": "main"
                        },
                        {
                            "type": "string",
                            "name": "timestamp",
                            "value": "2015-12-23T14:02:22.380405"
                        },
                        {
                            "type": "string",
                            "name": "trace_id",
                            "value": "016c97fd-87f3-40b2-9b55-e431156b694b"
                        }
                    ],
                "raw": {},
                "generated": "2015-12-23T10:41:38.406052",
                "event_type": "profiler.main",
                "message_id": "3256d9f1-48ba-4ac5-a50b-64fa42c6e264"},
            {
                "traits":
                    [
                        {
                            "type": "string",
                            "name": "base_id",
                            "value": "7253ca8c-33b3-4f84-b4f1-f5a4311ddfa4"
                        },
                        {
                            "type": "string",
                            "name": "db.params",
                            "value": "[]"
                        },
                        {
                            "type": "string",
                            "name": "db.statement",
                            "value": "SELECT 1"
                        },
                        {
                            "type": "string",
                            "name": "host",
                            "value": "ubuntu"
                        },
                        {
                            "type": "string",
                            "name": "name",
                            "value": "db-start"
                        },
                        {
                            "type": "string",
                            "name": "parent_id",
                            "value": "06320327-2c2c-45ae-923a-515de890276a"
                        },
                        {
                            "type": "string",
                            "name": "project",
                            "value": "keystone"
                        },
                        {
                            "type": "string",
                            "name": "service",
                            "value": "main"
                        },
                        {
                            "type": "string",
                            "name": "timestamp",
                            "value": "2015-12-23T14:02:22.395365"
                        },
                        {
                            "type": "string",
                            "name": "trace_id",
                            "value": "1baf1d24-9ca9-4f4c-bd3f-01b7e0c0735a"
                        }
                    ],
                "raw": {},
                "generated": "2015-12-23T10:41:38.984161",
                "event_type": "profiler.main",
                "message_id": "60368aa4-16f0-4f37-a8fb-89e92fdf36ff"
            },
            {
                "traits":
                    [
                        {
                            "type": "string",
                            "name": "base_id",
                            "value": "7253ca8c-33b3-4f84-b4f1-f5a4311ddfa4"
                        },
                        {
                            "type": "string",
                            "name": "host",
                            "value": "ubuntu"
                        },
                        {
                            "type": "string",
                            "name": "name",
                            "value": "db-stop"
                        },
                        {
                            "type": "string",
                            "name": "parent_id",
                            "value": "06320327-2c2c-45ae-923a-515de890276a"
                        },
                        {
                            "type": "string",
                            "name": "project",
                            "value": "keystone"
                        },
                        {
                            "type": "string",
                            "name": "service",
                            "value": "main"
                        },
                        {
                            "type": "string",
                            "name": "timestamp",
                            "value": "2015-12-23T14:02:22.415486"
                        },
                        {
                            "type": "string",
                            "name": "trace_id",
                            "value": "1baf1d24-9ca9-4f4c-bd3f-01b7e0c0735a"
                        }
                    ],
                "raw": {},
                "generated": "2015-12-23T10:41:39.019378",
                "event_type": "profiler.main",
                "message_id": "3fbeb339-55c5-4f28-88e4-15bee251dd3d"
            },
            {
                "traits":
                    [
                        {
                            "type": "string",
                            "name": "base_id",
                            "value": "7253ca8c-33b3-4f84-b4f1-f5a4311ddfa4"
                        },
                        {
                            "type": "string",
                            "name": "host",
                            "value": "ubuntu"
                        },
                        {
                            "type": "string",
                            "name": "method",
                            "value": "GET"
                        },
                        {
                            "type": "string",
                            "name": "name",
                            "value": "wsgi-start"
                        },
                        {
                            "type": "string",
                            "name": "parent_id",
                            "value": "7253ca8c-33b3-4f84-b4f1-f5a4311ddfa4"
                        },
                        {
                            "type": "string",
                            "name": "project",
                            "value": "keystone"
                        },
                        {
                            "type": "string",
                            "name": "service",
                            "value": "main"
                        },
                        {
                            "type": "string",
                            "name": "timestamp",
                            "value": "2015-12-23T14:02:22.427444"
                        },
                        {
                            "type": "string",
                            "name": "trace_id",
                            "value": "016c97fd-87f3-40b2-9b55-e431156b694b"
                        }
                    ],
                "raw": {},
                "generated": "2015-12-23T10:41:38.360409",
                "event_type": "profiler.main",
                "message_id": "57b971a9-572f-4f29-9838-3ed2564c6b5b"
            }
        ]

        expected = {"children": [
            {"children": [{"children": [],
                           "info": {"finished": 76,
                                    "host": "ubuntu",
                                    "meta.raw_payload.db-start": {},
                                    "meta.raw_payload.db-stop": {},
                                    "name": "db",
                                    "project": "keystone",
                                    "service": "main",
                                    "started": 56},
                           "parent_id": "06320327-2c2c-45ae-923a-515de890276a",
                           "trace_id": "1baf1d24-9ca9-4f4c-bd3f-01b7e0c0735a"}
                          ],
             "info": {"finished": 0,
                      "host": "ubuntu",
                      "meta.raw_payload.wsgi-start": {},
                      "name": "wsgi",
                      "project": "keystone",
                      "service": "main",
                      "started": 0},
             "parent_id": "7253ca8c-33b3-4f84-b4f1-f5a4311ddfa4",
             "trace_id": "06320327-2c2c-45ae-923a-515de890276a"},
            {"children": [],
             "info": {"finished": 41,
                      "host": "ubuntu",
                      "meta.raw_payload.wsgi-start": {},
                      "meta.raw_payload.wsgi-stop": {},
                      "name": "wsgi",
                      "project": "keystone",
                      "service": "main",
                      "started": 88},
             "parent_id": "7253ca8c-33b3-4f84-b4f1-f5a4311ddfa4",
             "trace_id": "016c97fd-87f3-40b2-9b55-e431156b694b"}],
            "info": {"finished": 88, "name": "total", "started": 0}}

        self.assertEqual(expected, ceilometer.parse_notifications(events))

    def test_get_notifications(self):
        mock_ceil_client = mock.MagicMock()
        results = [mock.MagicMock(), mock.MagicMock()]
        mock_ceil_client.events.list.return_value = results
        base_id = "10"

        result = ceilometer.get_notifications(mock_ceil_client, base_id)

        expected_filter = [{"field": "base_id", "op": "eq", "value": base_id}]
        mock_ceil_client.events.list.assert_called_once_with(expected_filter,
                                                             limit=100000)
        self.assertEqual(result, [results[0].to_dict(), results[1].to_dict()])



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from osprofiler import _utils as utils


class Notifier(object):

    def notify(self, info, context=None):
        """This method will be called on each notifier.notify() call.

        To add new drivers you should, create new subclass of this class and
        implement notify method.

        :param info:  Contains information about trace element.
                      In payload dict there are always 3 ids:
                      "base_id" - uuid that is common for all notifications
                                  related to one trace. Used to simplify
                                  retrieving of all trace elements from
                                  Ceilometer.
                      "parent_id" - uuid of parent element in trace
                      "trace_id" - uuid of current element in trace

                      With parent_id and trace_id it's quite simple to build
                      tree of trace elements, which simplify analyze of trace.

        :param context: request context that is mostly used to specify
                        current active user and tenant.
        """

    @staticmethod
    def factory(name, *args, **kwargs):
        for driver in utils.itersubclasses(Notifier):
            if name == driver.__name__:
                return driver(*args, **kwargs).notify

        raise TypeError("There is no driver, with name: %s" % name)



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from osprofiler import profiler


_DISABLED = False


def disable():
    """Disable tracing of all DB queries. Reduce a lot size of profiles."""
    global _DISABLED
    _DISABLED = True


def enable():
    """add_tracing adds event listeners for sqlalchemy."""

    global _DISABLED
    _DISABLED = False


def add_tracing(sqlalchemy, engine, name):
    """Add tracing to all sqlalchemy calls."""

    if not _DISABLED:
        sqlalchemy.event.listen(engine, "before_cursor_execute",
                                _before_cursor_execute(name))
        sqlalchemy.event.listen(engine, "after_cursor_execute",
                                _after_cursor_execute())


def _before_cursor_execute(name):
    """Add listener that will send trace info before query is executed."""

    def handler(conn, cursor, statement, params, context, executemany):
        info = {"db": {
            "statement": statement,
            "params": params}
        }
        profiler.start(name, info=info)

    return handler


def _after_cursor_execute():
    """Add listener that will send trace info after query is executed."""

    def handler(conn, cursor, statement, params, context, executemany):
        profiler.stop()

    return handler



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from osprofiler._notifiers import base


def _noop_notifier(info, context=None):
    """Do nothing on notify()."""


# NOTE(boris-42): By default we are using noop notifier.
__notifier = _noop_notifier


def notify(info):
    """Passes the profiling info to the notifier callable.

    :param info: dictionary with profiling information
    """
    __notifier(info)


def get():
    """Returns notifier callable."""
    return __notifier


def set(notifier):
    """Service that are going to use profiler should set callable notifier.

       Callable notifier is instance of callable object, that accept exactly
       one argument "info". "info" - is dictionary of values that contains
       profiling information.
    """
    global __notifier
    __notifier = notifier


def create(plugin_name, *args, **kwargs):
    """Create notifier based on specified plugin_name

    :param plugin_name: Name of plugin that creates notifier
    :param *args: args that will be passed to plugin init method
    :param **kwargs: kwargs that will be passed to plugin init method
    :returns: Callable notifier method
    :raises TypeError: In case of invalid name of plugin raises TypeError
    """
    return base.Notifier.factory(plugin_name, *args, **kwargs)



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from osprofiler._notifiers import base
from osprofiler.tests import test


class MessagingTestCase(test.TestCase):

    def test_init_and_notify(self):

        messaging = mock.MagicMock()
        context = "context"
        transport = "transport"
        project = "project"
        service = "service"
        host = "host"

        notify_func = base.Notifier.factory("Messaging", messaging, context,
                                            transport, project, service, host)

        messaging.Notifier.assert_called_once_with(
            transport, publisher_id=host, driver="messaging",
            topic="profiler", retry=0)

        info = {
            "a": 10
        }
        notify_func(info)

        expected_data = {"project": project, "service": service}
        expected_data.update(info)
        messaging.Notifier().info.assert_called_once_with(
            context, "profiler.%s" % service, expected_data)

        messaging.reset_mock()
        notify_func(info, context="my_context")
        messaging.Notifier().info.assert_called_once_with(
            "my_context", "profiler.%s" % service, expected_data)



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from osprofiler._notifiers import base


class Messaging(base.Notifier):

    def __init__(self, messaging, context, transport, project, service, host):
        """Init Messaging notify driver.

        """
        super(Messaging, self).__init__()
        self.messaging = messaging
        self.context = context
        self.project = project
        self.service = service

        self.notifier = messaging.Notifier(
            transport, publisher_id=host, driver="messaging",
            topic="profiler", retry=0)

    def notify(self, info, context=None):
        """Send notifications to Ceilometer via oslo.messaging notifier API.

        :param info:  Contains information about trace element.
                      In payload dict there are always 3 ids:
                      "base_id" - uuid that is common for all notifications
                                  related to one trace. Used to simplify
                                  retrieving of all trace elements from
                                  Ceilometer.
                      "parent_id" - uuid of parent element in trace
                      "trace_id" - uuid of current element in trace

                      With parent_id and trace_id it's quite simple to build
                      tree of trace elements, which simplify analyze of trace.

        :param context: request context that is mostly used to specify
                        current active user and tenant.
        """

        info["project"] = self.project
        info["service"] = self.service
        self.notifier.info(context or self.context,
                           "profiler.%s" % self.service, info)
